/* Function for getting Mem/Swap usage in percent. - FreeBSD specific code   */
/* Copyright(C) 2001 Seiichi SATO <sato@cvs-net.co.jp>,licensed under the GPL*/

#include <kvm.h>
#include <fcntl.h>
#include <sys/vmmeter.h>
#include <time.h>

static kvm_t *kvm_data = NULL;
static int pageshift;
static struct nlist nlst[] = {
  {"_cp_time", 0},
  {"_cnt", 0},
  {"_bufspace", 0},
  {0, 0}
};

/* initialize function */
int
mem_init (void)
{
  int pagesize = getpagesize ();
  pageshift = 0;

  while (pagesize > 1)
    {
      pageshift++;
      pagesize >>= 1;
    }

  kvm_data = kvm_open (NULL, NULL, NULL, O_RDONLY, "kvm_open");

  if (kvm_data == NULL)
    {
      puts ("Could not open kernel virtual memory");
      return -1;
    }
  kvm_nlist (kvm_data, nlst);

  if (nlst[0].n_type == 0 || nlst[1].n_type == 0 || nlst[2].n_type == 0)
    {
      puts ("Error extracting symbols");
      return -1;
    }
  /* drop setgid & setuid (the latter should not be there really) */
  seteuid (getuid ());
  setegid (getgid ());

  if (geteuid () != getuid () || getegid () != getgid ())
    {
      puts ("Unable to drop privileges");
      return -1;
    }
  return 0;
}


/* return mem/swap usage in percent 0 to 100 */
int
get_memusage (int *per_mem, int *per_swap)
{
  struct vmmeter vm;
  int bufspace;
  static int swap_firsttime = 1;
  static int swappgsin = -1;
  static int swappgsout = -1;
  static int swapmax = 0, swapused = 0;
  time_t cur_time;
  static time_t last_time_swap = 0;

  /* get mem usage */
  if (kvm_read (kvm_data, nlst[1].n_value, &vm, sizeof (vm)) != sizeof (vm))
    return -1;
  if (kvm_read (kvm_data, nlst[2].n_value, &bufspace, sizeof (bufspace)) !=
      sizeof (bufspace))
    return -1;

  /* get swap usage */
  /* only calculate when first time or when changes took place */
  /* do not call it more than 1 time per 2 seconds */
  /* otherwise it can eat up to 50% of CPU time on heavy swap activity */
  cur_time = time (NULL);
  if (swap_firsttime ||
      (((vm.v_swappgsin > swappgsin) || (vm.v_swappgsout > swappgsout)) &&
       cur_time > last_time_swap + 1))
    {

      struct kvm_swap swap;
      int n;

      swapmax = 0;
      swapused = 0;

      n = kvm_getswapinfo (kvm_data, &swap, 1, 0);
      if (n >= 0 && swap.ksw_total != 0)
	{
	  swapmax = swap.ksw_total;
	  swapused = swap.ksw_used;
	}

      swap_firsttime = 0;
      last_time_swap = cur_time;
    }
  swappgsin = vm.v_swappgsin;
  swappgsout = vm.v_swappgsout;

  /* calc mem/swap usage in percent */
  *per_mem =
    100 * (1 - ((double) vm.v_free_count / (double) vm.v_page_count));
  *per_swap = 100 * (double) swapused / swapmax;
  return 0;
}
