/*    WMMemMon - WindowMaker Mem/swap Monitor dockapp                      */
/*    Copyright (C) 2001  Seiichi SATO <sato@cvs-net.co.jp>                */

/*    This program is free software; you can redistribute it and/or modify */
/*    it under the terms of the GNU General Public License as published by */
/*    the Free Software Foundation; either version 2 of the License, or    */
/*    (at your option) any later version.                                  */

/*    This program is distributed in the hope that it will be useful,      */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*    GNU General Public License for more details.                         */

/*    You should have received a copy of the GNU General Public License    */
/*    along with this program; if not, write to the Free Software          */
/*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <sys/time.h>
#include <dockapp.h>
#include "mem_common.h"
#include "backdrop_on.xpm"
#include "backdrop_off.xpm"
#include "parts_mem.xpm"
#include "parts_swap.xpm"

/* global data */
char *displayName = "";
GC gc;
Pixmap pixmap;
Pixmap backdrop_on;
Pixmap backdrop_off;
Pixmap parts_mem;
Pixmap parts_swap;
Pixmap mask;
int per_mem, per_swap;
int update_interval = 1;
int color_index;

typedef enum
{
  BackLightOn,
  BackLightOff,
}
PanelMode;
PanelMode panelmode = BackLightOff;

static DAProgramOption options[] = {
  {"-d", "--display", "display to use", DOString, False, {&displayName}},
  {"-bl", "--backlight", "switches with Back-Light on", DONone, False, {NULL}},
  {"-i", "--interval", "number of secs between updates (1 is default)",
   DONatural, False, {&update_interval}},
};

/* prototypes */
static void changePanel (int button, int state, int x, int y);
static void refresh (void);
static void paintdock (int per_mem, int per_swap);

int
main (int argc, char **argv)
{
  static unsigned d_width = 58;
  static unsigned d_height = 58;
  DACallbacks callbacks =
    { NULL, changePanel, NULL, NULL, NULL, NULL, refresh };

  char *package, *version;
  package = malloc ((strlen (PACKAGE) + 1) * sizeof (char));
  sprintf (package, "%s", PACKAGE);
  version = malloc ((strlen (package) + strlen (" version ")
		     + strlen (VERSION) + 1) * sizeof (char));
  sprintf (version, "%s version %s", package, VERSION);

  /* Parse CommandLine */
  DAParseArguments (argc, argv, options,
		    sizeof (options) / sizeof (DAProgramOption),
		    package, version);
  if (options[1].used)
    {
      panelmode = BackLightOn;
    }

  /* Initialize Application */
  if (mem_init ())
    exit (-1);
  DAInitialize (displayName, "wmmemmon", d_width, d_height, argc, argv);
  DAMakePixmapFromData (backdrop_on_xpm, &backdrop_on, &mask, &d_width,
			&d_height);
  DAMakePixmapFromData (backdrop_off_xpm, &backdrop_off, NULL, &d_width,
			&d_height);
  DAMakePixmapFromData (parts_mem_xpm, &parts_mem, NULL, &d_width, &d_height);
  DAMakePixmapFromData (parts_swap_xpm, &parts_swap, NULL, &d_width,
			&d_height);
  DASetShape (mask);
  pixmap = DAMakePixmap ();
  gc = DefaultGC (DADisplay, DefaultScreen (DADisplay));

  /* Set WindowTitle for AS Wharf */
  XStoreName (DADisplay, DAWindow, "wmmemmon");

  /* Initialize Callbacks */
  DASetCallbacks (&callbacks);
  DASetTimeout (1000 * update_interval);

  /* Initialize pixmap */
  paintdock (0, 0);
  DASetPixmap (pixmap);
  DAShow ();

  /* Main loop */
  DAEventLoop ();

  return 0;
}

/* called when mouse button pressed */
static void
changePanel (int button, int state, int x, int y)
{
  switch (panelmode)
    {
    case BackLightOff:
      panelmode = BackLightOn;
      break;
    case BackLightOn:
      panelmode = BackLightOff;
      break;
    }

  paintdock (per_mem, per_swap);
}

/* called by timer */
static void
refresh (void)
{
  if (get_memusage (&per_mem, &per_swap))
    exit (-1);
  paintdock (per_mem, per_swap);
}

static void
paintdock (int mem, int swap)
{
  int y_mem = 0;
  int y_swap = 0;

  mem /= 10;
  swap /= 10;
  if (mem > 10)
    mem=10;

  /* clear */
  switch (panelmode)
    {
    case BackLightOn:
      XCopyArea (DADisplay, backdrop_on, pixmap, gc, 0, 0, 58, 58, 0, 0);
      y_mem = 40;
      y_swap = 21;
      break;
    case BackLightOff:
      XCopyArea (DADisplay, backdrop_off, pixmap, gc, 0, 0, 58, 58, 0, 0);
      break;
    }

  /* draw Mem usage - outside  circle */
  if (mem != 0 && mem < 5)
    XCopyArea (DADisplay, parts_mem, pixmap, gc, (mem - 1) * 19, y_mem, 19,
	       40, 30, 9);
  if (mem >= 5)
    XCopyArea (DADisplay, parts_mem, pixmap, gc, 76, y_mem, 19, 40, 30, 9);
  if (mem > 5)
    XCopyArea (DADisplay, parts_mem, pixmap, gc, 95 + (mem - 6) * 20, y_mem,
	       20, 40, 9, 9);

  /* draw Swap usage - inside  circle */
  if (swap != 0 && swap < 5)
    {
      XCopyArea (DADisplay, parts_swap, pixmap, gc, (swap - 1) * 10, y_swap,
		 4, 21, 30, 19);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 4 + (swap - 1) * 10,
		 y_swap + 2, 3, 16, 34, 21);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 7 + (swap - 1) * 10,
		 y_swap + 4, 2, 13, 37, 23);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 9 + (swap - 1) * 10,
		 y_swap + 8, 1, 4, 39, 27);
    }
  if (swap >= 5)
    {
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 40, y_swap, 4, 21, 30,
		 19);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 44, y_swap + 2, 3, 16, 34,
		 21);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 47, y_swap + 4, 2, 13, 37,
		 23);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 49, y_swap + 8, 1, 4, 39,
		 27);
    }
  if (swap > 5)
    {
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 50 + (swap - 6) * 10,
		 y_swap + 5, 2, 11, 19, 24);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 52 + (swap - 6) * 10,
		 y_swap + 2, 2, 17, 21, 21);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 54 + (swap - 6) * 10,
		 y_swap + 1, 2, 19, 23, 20);
      XCopyArea (DADisplay, parts_swap, pixmap, gc, 56 + (swap - 6) * 10,
		 y_swap, 4, 21, 25, 19);
    }

  /* copy to DAWindow(icon-window) */
  XCopyArea (DADisplay, pixmap, DAWindow, gc, 0, 0, 64, 64, 0, 0);
}
